/******************************************************************************
 *
 * File:        client.h
 *
 * Description: This file contains the declaration of the Client class.
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef CLIENT_H
#define CLIENT_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include "controller.h"
#include "details.h"
#include <QObject>
#include <QtQml/qqmlregistration.h>

namespace WindowManager {
// Q_NAMESPACE
namespace Core {

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief The Client class contains meta data about a process, from
     * .desktop files, and a controller to control the state of the process.
     */
    class Client : public QObject
    {
        Q_OBJECT

        Q_PROPERTY(QString name READ name CONSTANT)
        Q_PROPERTY(uint pid READ pid CONSTANT)
        Q_PROPERTY(Details *details READ details CONSTANT)
        Q_PROPERTY(ExecutionState executionState READ executionState NOTIFY executionStateChanged)
        QML_ELEMENT

    public:
        enum ExecutionState
        {
            ES_START,
            ES_STARTING,
            ES_RUNNING,
            ES_PAUSE,
            ES_PAUSING,
            ES_PAUSED,
            ES_RESUME,
            ES_RESUMING,
            ES_STOP,
            ES_STOPPING,
            ES_STOPPED
        };
        Q_ENUM(ExecutionState)

        /**
         * @brief Default constructor of the class
         */
        Client(QObject *parent = nullptr);

        /**
         * @brief Constructor to create an object representing a process
         * @param pid The PID of the process
         * @param binName The binary name of the process
         * @param *details Pointer to a Details object describing the process
         */
        Client(uint pid, QString binName, Details *details, QObject *parent = nullptr);

        /**
         * @brief Returns the process id (pid) of the client when running, otherwise 0
         * @return Unsigned integer with the process id
         */
        uint pid();

        /**
         * @brief Returns the details object describing the client
         * @return Details The details object
         */
        Details *details();

        /**
         * @brief Get the current execution state of the client
         * @return ExecutionState Current execution state
         */
        ExecutionState executionState();

        /**
         * @brief Request a that the client should change its execution state
         * @param value The requested execution state
         * @return bool if the request was successful
         */
        Q_INVOKABLE bool requestExecutionState(ExecutionState value);

        /**
         * @brief Quit a client based on its PID
         */
        void quitClient();

        /**
         * @brief Terminates a client based on its PID
         */
        void terminateClient();

    signals:
        /**
         * @brief Signal to notify that the execution state have changed
         */
        void executionStateChanged();

    private:
        void startClient();
        void pauseClient();
        void resumeClient();

        QString name();
        void setExecutionState(ExecutionState value);

        uint m_pid;
        QString m_binName;
        Details *m_details;
        Controller *m_controller;
        ExecutionState m_executionState;
    };

} // Core
} // WindowManager

#endif // CLIENT_H
