/******************************************************************************
 *
 * File:        json_helpers.h
 *
 * Description: This file contains helper functions that are used to parse JSON
 * configuration files.
 *
 * JSON file handling is based on the example
 * https://doc.qt.io/qt-6.4/qtcore-serialization-savegame-example.html
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef JSON_HELPERS_H
#define JSON_HELPERS_H

#include <QJsonArray>
#include <QJsonObject>

/******************************************************************************
 * Static Methods
 ******************************************************************************/

/**
 * @brief Returns an integer converted from JSON object
 * @return int of the value if successful, otherwise 0
 */
static int getJsonInteger(const QJsonObject &json, QString identifier) {
    // 'isDouble' is the way to detect 'numeric' value, eventough integer is
    // expected in this case
    if (json.contains(identifier) && json[identifier].isDouble()) {
        int result = json[identifier].toInt();
        return result;
    }
    else {
        qWarning() << "JSON integer value not found: " << identifier;
        return 0;
    }
}

/**
 * @brief Returns a double converted from JSON object
 * @return double of the value if successful, otherwise 0
 */
static double getJsonDouble(const QJsonObject &json, QString identifier) {
    if (json.contains(identifier) && json[identifier].isDouble()) {
        double result = json[identifier].toDouble();
        return result;
    }
    else {
        qWarning() << "JSON double value not found: " << identifier;
        return 0;
    }
}

/**
 * @brief Returns a string converted from JSON object
 * @return QString of the value if successful, otherwise empty ("")
 */
static QString getJsonString(const QJsonObject &json, QString identifier) {
    if (json.contains(identifier) && json[identifier].isString()) {
        QString result = json[identifier].toString();
        return result;
    }
    else {
        qWarning() << "JSON string value not found: " << identifier;
        return "";
    }
}

/**
 * @brief Returns a QJsonArray converted from JSON object
 * @return QJsonArray of the value if successful, otherwise an empty array
 */
static QJsonArray getJsonArray(const QJsonObject &json, QString identifier) {
    if (json.contains(identifier) && json[identifier].isArray()) {
        QJsonArray result = json[identifier].toArray();
        return result;
    }
    else {
        qWarning() << "JSON array value not found: " << identifier;
        return QJsonArray();
    }
}

#endif // JSON_HELPERS_H
