/******************************************************************************
 *
 * File:        manager.h
 *
 * Description: This file contains the declaration of the Manager class.
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef MANAGER_H
#define MANAGER_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include "client.h"
#include "details.h"
#include "layout.h"
#include "unixsignaling.h"
#include <QDir>
#include <QList>
#include <QObject>
#include <QProcess>
#include <QQmlEngine>
#include <QRectF>
#include <QRegularExpression>
#include <QtQml/qqmlregistration.h>

using namespace WindowManager::Core;

namespace WindowManager {
namespace Core {

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief The Manager class is the main backend of the Window Manager. This
     * class keeps track of all registered clients, all registered desktop
     * entries and all registered layouts.
     */
    class Manager : public QObject
    {
        Q_OBJECT

        Q_PROPERTY(QList<Layout *> layouts READ layouts CONSTANT)
        Q_PROPERTY(int selectedLayoutIndex READ selectedLayoutIndex WRITE setSelectedLayoutIndex NOTIFY selectedLayoutIndexChanged)
        Q_PROPERTY(QList<Region *> currentRegions READ currentRegions NOTIFY currentRegionsChanged)
        Q_PROPERTY(QList<Client *> executableClients READ executableClients CONSTANT)

        QML_ELEMENT
        QML_UNCREATABLE("Core Backend class")

    public:
        /**
         * @brief Default constructor of the class
         */
        Manager(QObject *parent = nullptr);

        /**
         * @brief Overload constructor of the class that accepts custom layout
         * and client detail directories
         */
        Manager(QString layoutsDir, QString clientsDir, bool debugMode, QObject *parent = nullptr);

        /**
         * @brief Destructor - forwards close event to connected apps
         */
        ~Manager();

        /**
         * @brief Register a new client in the manager
         */
        Q_INVOKABLE Client *registerClient(uint pid);

        /**
         * @brief Unregister a client from the manager
         * @param pid The process id of the client to unregister
         * @return true if successful unregister, false if unsuccessful unregister or pid not found
         */
        Q_INVOKABLE bool unregisterClient(uint pid);

        Client *createClient(Details *clientDetails);

        /**
         * @brief Get region with regionId from current layout
         */
        Q_INVOKABLE Region *getRegion(QString regionId, QList<Region *> *subRegions = nullptr);

        /**
         * @brief Get list of all regions from current layout
         * @return List of all regions from current layout
         */
        QList<Region *> &currentRegions();

        /**
         * @brief Get list of all executable clients
         * @return List of all executable clients
         */
        QList<Client *> executableClients();

        /**
         * @brief Get region rect from current regions by regionId in normalized coordinates to layout
         */
        Q_INVOKABLE QRectF getRegionRect(QString regionId, QList<Region *> *subRegions = nullptr);

        /**
         * @brief Get list of all layouts
         * @return List of all layouts
         */
        QList<Layout *> layouts();

        /**
         * @brief Get selected layout index
         * @return Current selected layout index
         */
        int selectedLayoutIndex();

        /**
         * @brief Set selected layout index and update list of current regions 
         * to the regions in the selected layout
         * @param value The layout index
         */
        void setSelectedLayoutIndex(int value);

        /**
         * @brief Sets which layout to use
         * @param layoutId ID of layout to use
         * @return True if layout successfully set, False otherwise (typically because ID does not exist)
         */
        Q_INVOKABLE bool setCurrentLayout(QString layoutId);

        /**
         * @brief Get Z value from Region with regionId in currentRegions
         * @param regionId ID of region to retrieve the Z value from
         * @return Z value if region with regionID is found, if not found return 0
         */
        Q_INVOKABLE int getRegionZ(QString regionId, QList<Region *> *subRegions = nullptr);

    signals:
        void selectedLayoutIndexChanged();
        void currentRegionsChanged();
        void quitWindowManager();

    private:
        void initDetails(QString dir);
        void initLayouts(QString dir);
        QString getClientName(uint pid);
        QString getClientPath(uint pid);
        QString getClientCommand(uint pid);
        Details *addDetail(QString binaryName, QString binaryPath, QString execCommand);
        bool getClientDetails(QString clientName, Details **outDetail);
        QString runSystemCommand(QString program, QStringList arguments, QString &outError);
        void quitAllClients();
        void terminateAllClients();

        QList<Details *> m_details;
        QMap<uint, Client *> m_clients;
        QList<Layout *> m_layouts;
        QList<Region *> m_currentRegions;
        int m_selectedLayoutIndex;
        UnixSignaling *m_unixSignaling;

        bool m_debugMode;
    };
} // Core
} // WindowManager

#endif // MANAGER_H
